@@ -480,7 +480,6 @@ i830_memory *i830_allocate_framebuffer(ScrnInfoPtr scrn)
 {
 	intel_screen_private *intel = intel_get_screen_private(scrn);
 	unsigned int pitch = scrn->displayWidth * intel->cpp;
-	unsigned long minspace;
 	long size, fb_height;
 	int flags;
 	i830_memory *front_buffer = NULL;
@@ -493,12 +492,6 @@ i830_memory *i830_allocate_framebuffer(ScrnInfoPtr scrn)
 	 */
 	fb_height = scrn->virtualY;
 
-	/* Calculate how much framebuffer memory to allocate.  For the
-	 * initial allocation, calculate a reasonable minimum.  This is
-	 * enough for the virtual screen size.
-	 */
-	minspace = pitch * scrn->virtualY;
-
 	size = ROUND_TO_PAGE(pitch * fb_height);
 
 	if (intel->tiling && IsTileable(scrn, pitch))
@@ -523,8 +516,7 @@ i830_memory *i830_allocate_framebuffer(ScrnInfoPtr scrn)
 		return NULL;
 	}
 
-	i830_set_max_gtt_map_size(scrn);
-	i830_set_max_tiling_size(scrn);
+	i830_set_gem_max_sizes(scrn);
 
 	return front_buffer;
 }
@@ -594,8 +586,7 @@ Bool i830_bind_all_memory(ScrnInfoPtr scrn)
 		drmmode_crtc_set_cursor_bo(xf86_config->crtc[i],
 					   intel->cursor_mem_argb[i]->bo);
 
-	i830_set_max_gtt_map_size(scrn);
-	i830_set_max_tiling_size(scrn);
+	i830_set_gem_max_sizes(scrn);
 
 	if (intel->front_buffer)
 		scrn->fbOffset = intel->front_buffer->offset;
@@ -644,7 +635,7 @@ void i830_free_xvmc_buffer(ScrnInfoPtr scrn, i830_memory * buffer)
 
 #endif
 
-void i830_set_max_gtt_map_size(ScrnInfoPtr scrn)
+static void i830_set_max_gtt_map_size(ScrnInfoPtr scrn)
 {
 	intel_screen_private *intel = intel_get_screen_private(scrn);
 	struct drm_i915_gem_get_aperture aperture;
@@ -665,7 +656,7 @@ void i830_set_max_gtt_map_size(ScrnInfoPtr scrn)
 	}
 }
 
-void i830_set_max_tiling_size(ScrnInfoPtr scrn)
+static void i830_set_max_tiling_size(ScrnInfoPtr scrn)
 {
 	intel_screen_private *intel = intel_get_screen_private(scrn);
 	struct drm_i915_gem_get_aperture aperture;
@@ -685,3 +676,9 @@ void i830_set_max_tiling_size(ScrnInfoPtr scrn)
 			intel->max_tiling_size /= 2;
 	}
 }
+
+void i830_set_gem_max_sizes(ScrnInfoPtr scrn)
+{
+	i830_set_max_gtt_map_size(scrn);
+	i830_set_max_tiling_size(scrn);
+}
